@@ -15,6 +15,7 @@
 
 # Manage jobs in Jenkins server
 
+import errno
 import os
 import sys
 import hashlib
@@ -79,7 +80,11 @@ def deep_format(obj, paramdict):
     # example, is problematic).
     if isinstance(obj, str):
         try:
-            ret = obj.format(**paramdict)
+            result = re.match('^{obj:(?P<key>\w+)}$', obj)
+            if result is not None:
+                ret = paramdict[result.group("key")]
+            else:
+                ret = obj.format(**paramdict)
         except KeyError as exc:
             missing_key = exc.message
             desc = "%s parameter missing to format %s\nGiven: %s" % (
@@ -98,32 +103,34 @@ def deep_format(obj, paramdict):
     return ret
 
 
-def matches(what, where):
+def matches(what, glob_patterns):
     """
-    Checks if the given string matches against the given list of glob patterns
+    Checks if the given string, ``what``, matches any of the glob patterns in
+    the iterable, ``glob_patterns``
 
-    :arg str what: String that we want to test if matches
-    :arg list where: list of glob patters to match
+    :arg str what: String that we want to test if it matches a pattern
+    :arg iterable glob_patterns: glob patterns to match (list, tuple, set,
+    etc.)
     """
-    for pattern in where:
-        if re.match(fnmatch.translate(pattern), what):
-            return True
-    return False
+    return any(fnmatch.fnmatch(what, glob_pattern)
+               for glob_pattern in glob_patterns)
 
 
 class YamlParser(object):
     def __init__(self, config=None):
-        self.registry = ModuleRegistry(config)
         self.data = {}
         self.jobs = []
+        self.config = config
+        self.registry = ModuleRegistry(self.config)
 
-    def parse(self, fn):
-        data = yaml.load(open(fn))
+    def parse_fp(self, fp):
+        data = yaml.load(fp)
         if data:
             if not isinstance(data, list):
                 raise JenkinsJobsException(
                     "The topmost collection in file '{fname}' must be a list,"
-                    " not a {cls}".format(fname=fn, cls=type(data)))
+                    " not a {cls}".format(fname=getattr(fp, 'name', fp),
+                                          cls=type(data)))
             for item in data:
                 cls, dfn = item.items()[0]
                 group = self.data.get(cls, {})
@@ -141,6 +148,10 @@ class YamlParser(object):
                 group[name] = dfn
                 self.data[cls] = group
 
+    def parse(self, fn):
+        with open(fn) as fp:
+            self.parse_fp(fp)
+
     def getJob(self, name):
         job = self.data.get('job', {}).get(name, None)
         if not job:
@@ -279,8 +290,18 @@ class YamlParser(object):
 
     def getXMLForJob(self, data):
         kind = data.get('project-type', 'freestyle')
-        data["description"] = (data.get("description", "") +
-                               self.get_managed_string()).lstrip()
+        if self.config:
+            keep_desc = self.config.getboolean('job_builder',
+                                               'keep_descriptions')
+        else:
+            keep_desc = False
+        if keep_desc:
+            description = data.get("description", None)
+        else:
+            description = data.get("description", '')
+        if description is not None:
+            data["description"] = description + \
+                self.get_managed_string().lstrip()
         for ep in pkg_resources.iter_entry_points(
                 group='jenkins_jobs.projects', name=kind):
             Mod = ep.load()
@@ -339,7 +360,7 @@ class ModuleRegistry(object):
 
         :arg string component_type: the name of the component
           (e.g., `builder`)
-        :arg YAMLParser parser: the global YMAL Parser
+        :arg YAMLParser parser: the global YAML Parser
         :arg Element xml_parent: the parent XML element
         :arg dict template_data: values that should be interpolated into
           the component definition
@@ -359,7 +380,7 @@ class ModuleRegistry(object):
             .component_list_type
 
         if isinstance(component, dict):
-            # The component is a sigleton dictionary of name: dict(args)
+            # The component is a singleton dictionary of name: dict(args)
             name, component_data = component.items()[0]
             if template_data:
                 # Template data contains values that should be interpolated
@@ -510,13 +531,19 @@ class Builder(object):
         self.ignore_cache = ignore_cache
 
     def load_files(self, fn):
+        self.parser = YamlParser(self.global_config)
+
+        if hasattr(fn, 'read'):
+            self.parser.parse_fp(fn)
+            return
+
         if os.path.isdir(fn):
             files_to_process = [os.path.join(fn, f)
                                 for f in os.listdir(fn)
                                 if (f.endswith('.yml') or f.endswith('.yaml'))]
         else:
             files_to_process = [fn]
-        self.parser = YamlParser(self.global_config)
+
         for in_file in files_to_process:
             logger.debug("Parsing YAML file {0}".format(in_file))
             self.parser.parse(in_file)
@@ -552,8 +579,8 @@ class Builder(object):
         for job in jobs:
             self.delete_job(job['name'])
 
-    def update_job(self, fn, names=None, output_dir=None):
-        self.load_files(fn)
+    def update_job(self, input_fn, names=None, output=None):
+        self.load_files(input_fn)
         self.parser.generateXML(names)
 
         self.parser.jobs.sort(lambda a, b: cmp(a.name, b.name))
@@ -561,13 +588,32 @@ class Builder(object):
         for job in self.parser.jobs:
             if names and not matches(job.name, names):
                 continue
-            if output_dir:
-                if names:
-                    print job.output()
+            if output:
+                if hasattr(output, 'write'):
+                    # `output` is a file-like object
+                    logger.debug("Writing XML to '{0}'".format(output))
+                    try:
+                        output.write(job.output())
+                    except IOError as exc:
+                        if exc.errno == errno.EPIPE:
+                            # EPIPE could happen if piping output to something
+                            # that doesn't read the whole input (e.g.: the UNIX
+                            # `head` command)
+                            return
+                        raise
                     continue
-                fn = os.path.join(output_dir, job.name)
-                logger.debug("Writing XML to '{0}'".format(fn))
-                f = open(fn, 'w')
+
+                output_dir = output
+
+                try:
+                    os.makedirs(output_dir)
+                except OSError:
+                    if not os.path.isdir(output_dir):
+                        raise
+
+                output_fn = os.path.join(output_dir, job.name)
+                logger.debug("Writing XML to '{0}'".format(output_fn))
+                f = open(output_fn, 'w')
                 f.write(job.output())
                 f.close()
                 continue
