@@ -153,9 +153,9 @@ def least_load(parser, xml_parent, data):
 
     :arg bool disabled: whether or not leastload is disabled (default True)
 
-    Example::
+    Example:
 
-    .. literalinclude:: /../../tests/properties/fixtures/least_load002.yaml
+    .. literalinclude:: /../../tests/properties/fixtures/least-load002.yaml
     """
     least = XML.SubElement(xml_parent,
                            'org.bstick12.jenkinsci.plugins.leastload.'
@@ -475,6 +475,36 @@ def build_blocker(parser, xml_parent, data):
     XML.SubElement(blocker, 'blockingJobs').text = jobs
 
 
+def copyartifact(parser, xml_parent, data):
+    """yaml: copyartifact
+    Specify a list of projects that have access to copy the artifacts of
+    this project.
+
+    Requires the Jenkins `Copy Artifact plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/Copy+Artifact+Plugin>`_
+
+    :arg string projects: comma separated list of projects that can copy
+        artifacts of this project. Wild card character '*' is available.
+
+
+    Example:
+
+    .. literalinclude:: \
+            /../../tests/properties/fixtures/copyartifact.yaml
+
+    """
+    copyartifact = XML.SubElement(xml_parent,
+                                  'hudson.plugins.'
+                                  'copyartifact.'
+                                  'CopyArtifactPermissionProperty',
+                                  plugin='copyartifact')
+    if not data or not data.get('projects', None):
+        raise JenkinsJobsException("projects string must exist and "
+                                   "not be empty")
+    projectlist = XML.SubElement(copyartifact, 'projectNameList')
+    XML.SubElement(projectlist, 'string').text = data.get('projects')
+
+
 def batch_tasks(parser, xml_parent, data):
     """yaml: batch-tasks
     Batch tasks can be tasks for events like releases, integration, archiving,
@@ -533,6 +563,36 @@ def heavy_job(parser, xml_parent, data):
         data.get('weight', 1))
 
 
+def slave_utilization(parser, xml_parent, data):
+    """yaml: slave-utilization
+    This plugin allows you to specify the percentage of a slave's capacity a
+    job wants to use.
+
+    Requires the Jenkins `Slave Utilization Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/Slave+Utilization+Plugin>`_
+
+    :arg int slave-percentage: Specify the percentage of a slave's execution
+        slots that this job should occupy (default: 0)
+    :arg bool single-instance-per-slave: Control whether concurrent instances
+        of this job will be permitted to run in parallel on a single slave
+        (default: False)
+
+    Example:
+
+    .. literalinclude:: \
+            /../../tests/properties/fixtures/slave-utilization1.yaml
+    """
+    utilization = XML.SubElement(
+        xml_parent, 'com.suryagaddipati.jenkins.SlaveUtilizationProperty')
+    percent = int(data.get('slave-percentage', 0))
+    XML.SubElement(utilization, 'needsExclusiveAccessToNode'
+                   ).text = 'true' if percent else 'false'
+    XML.SubElement(utilization, 'slaveUtilizationPercentage'
+                   ).text = str(percent)
+    XML.SubElement(utilization, 'singleInstancePerSlave').text = str(
+        data.get('single-instance-per-slave', False)).lower()
+
+
 def delivery_pipeline(parser, xml_parent, data):
     """yaml: delivery-pipeline
     Requires the Jenkins `Delivery Pipeline Plugin.
@@ -554,6 +614,27 @@ def delivery_pipeline(parser, xml_parent, data):
     XML.SubElement(pipeline, 'taskName').text = data.get('task', '')
 
 
+def zeromq_event(parser, xml_parent, data):
+    """yaml: zeromq-event
+    This is a Jenkins plugin that will publish Jenkins Job run events
+    (start, complete, finish) to a ZMQ PUB socket.
+
+    Requires the Jenkins `ZMQ Event Publisher.
+    <https://git.openstack.org/cgit/openstack-infra/zmq-event-publisher>`_
+
+    Example:
+
+    .. literalinclude:: \
+            /../../tests/properties/fixtures/zeromq-event.yaml
+
+    """
+
+    zmq_event = XML.SubElement(xml_parent,
+                               'org.jenkinsci.plugins.'
+                               'ZMQEventPublisher.HudsonNotificationProperty')
+    XML.SubElement(zmq_event, 'enabled').text = 'true'
+
+
 class Properties(jenkins_jobs.modules.base.Base):
     sequence = 20
 
