@@ -44,6 +44,13 @@ def git(self, xml_parent, data):
     :arg str credentials-id: ID of credentials to use to connect (optional)
     :arg str refspec: refspec to fetch
     :arg str name: name to fetch
+    :arg list(str) remotes: list of remotes to set up (optional, only needed if
+      multiple remotes need to be set up)
+
+        :Remote: * **url** (`string`) - url of remote repo
+                 * **refspec** (`string`) - refspec to fetch (optional)
+                 * **credentials-id** - ID of credentials to use to connect
+                     (optional)
     :arg list(str) branches: list of branch specifiers to build
     :arg list(str) excluded-users: list of users to ignore revisions from
       when polling for changes. (if polling is enabled)
@@ -79,14 +86,17 @@ def git(self, xml_parent, data):
     :arg str git-config-email: Configure email for Git clone
 
     :browser values:
-        :githubweb:
-        :fisheye:
+        :auto:
         :bitbucketweb:
+        :cgit:
+        :fisheye:
         :gitblit:
+        :githubweb:
         :gitlab:
         :gitoriousweb:
         :gitweb:
         :redmineweb:
+        :stash:
         :viewgit:
 
     :choosing-strategy values:
@@ -133,16 +143,27 @@ def git(self, xml_parent, data):
                          'scm', {'class': 'hudson.plugins.git.GitSCM'})
     XML.SubElement(scm, 'configVersion').text = '2'
     user = XML.SubElement(scm, 'userRemoteConfigs')
-    huser = XML.SubElement(user, 'hudson.plugins.git.UserRemoteConfig')
-    XML.SubElement(huser, 'name').text = data.get('name', 'origin')
-    if 'refspec' in data:
-        refspec = data['refspec']
-    else:
-        refspec = '+refs/heads/*:refs/remotes/origin/*'
-    XML.SubElement(huser, 'refspec').text = refspec
-    XML.SubElement(huser, 'url').text = data['url']
-    if 'credentials-id' in data:
-        XML.SubElement(huser, 'credentialsId').text = data['credentials-id']
+    if 'remotes' not in data:
+        data['remotes'] = [{data.get('name', 'origin'): data.copy()}]
+    for remoteData in data['remotes']:
+        huser = XML.SubElement(user, 'hudson.plugins.git.UserRemoteConfig')
+        remoteName = remoteData.keys()[0]
+        XML.SubElement(huser, 'name').text = remoteName
+        remoteParams = remoteData.values()[0]
+        if 'refspec' in remoteParams:
+            refspec = remoteParams['refspec']
+        else:
+            refspec = '+refs/heads/*:refs/remotes/' + remoteName + '/*'
+        XML.SubElement(huser, 'refspec').text = refspec
+        if 'url' in remoteParams:
+            remoteURL = remoteParams['url']
+        else:
+            raise JenkinsJobsException('Must specify a url for git remote \"' +
+                                       remoteName + '"')
+        XML.SubElement(huser, 'url').text = remoteURL
+        if 'credentials-id' in remoteParams:
+            credentialsId = remoteParams['credentials-id']
+            XML.SubElement(huser, 'credentialsId').text = credentialsId
     xml_branches = XML.SubElement(scm, 'branches')
     branches = data.get('branches', ['**'])
     for branch in branches:
@@ -189,22 +210,23 @@ def git(self, xml_parent, data):
         XML.SubElement(scm, 'localBranch').text = data['local-branch']
 
     browser = data.get('browser', 'auto')
-    browserdict = {'githubweb': 'GithubWeb',
-                   'fisheye': 'FisheyeGitRepositoryBrowser',
+    browserdict = {'auto': 'auto',
                    'bitbucketweb': 'BitbucketWeb',
                    'cgit': 'CGit',
+                   'fisheye': 'FisheyeGitRepositoryBrowser',
                    'gitblit': 'GitBlitRepositoryBrowser',
+                   'githubweb': 'GithubWeb',
                    'gitlab': 'GitLab',
                    'gitoriousweb': 'GitoriousWeb',
                    'gitweb': 'GitWeb',
                    'redmineweb': 'RedmineWeb',
-                   'viewgit': 'ViewGitWeb',
-                   'auto': 'auto'}
+                   'stash': 'Stash',
+                   'viewgit': 'ViewGitWeb'}
     if browser not in browserdict:
+        valid = sorted(browserdict.keys())
         raise JenkinsJobsException("Browser entered is not valid must be one "
-                                   "of: githubweb, fisheye, bitbucketweb, "
-                                   "cgit, gitblit, gitlab, gitoriousweb, "
-                                   "gitweb, redmineweb, viewgit, or auto")
+                                   "of: %s or %s." % (", ".join(valid[:-1]),
+                                                      valid[-1]))
     if browser != 'auto':
         bc = XML.SubElement(scm, 'browser', {'class':
                             'hudson.plugins.git.browser.' +
@@ -470,6 +492,44 @@ def tfs(self, xml_parent, data):
                                                   'Browser'})
 
 
+def workspace(self, xml_parent, data):
+    """yaml: workspace
+    Specifies the cloned workspace for this job to use as a SCM source.
+    Requires the Jenkins `Clone Workspace SCM Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/Clone+Workspace+SCM+Plugin>`_
+
+    The job the workspace is cloned from must be configured with an
+    clone-workspace publisher
+
+    :arg str parent-job: The name of the parent job to clone the
+        workspace from.
+    :arg str criteria: Set the criteria to determine what build of the parent
+        project to use. Can be one of 'Any', 'Not Failed' or 'Successful'.
+        (default: Any)
+
+
+    Example:
+
+    .. literalinclude:: /../../tests/scm/fixtures/workspace001.yaml
+    """
+
+    workspace = XML.SubElement(xml_parent, 'scm', {'class': 'hudson.plugins.'
+                               'cloneworkspace.CloneWorkspaceSCM'})
+    XML.SubElement(workspace, 'parentJobName').text = str(
+        data.get('parent-job', ''))
+
+    criteria_list = ['Any', 'Not Failed', 'Successful']
+
+    criteria = data.get('criteria', 'Any').title()
+
+    if 'criteria' in data and criteria not in criteria_list:
+        raise JenkinsJobsException(
+            'clone-workspace criteria must be one of: '
+            + ', '.join(criteria_list))
+    else:
+        XML.SubElement(workspace, 'criteria').text = criteria
+
+
 class SCM(jenkins_jobs.modules.base.Base):
     sequence = 30
 
